#!/usr/bin/perl
# filename:          projectManager.vhd
# created by:        Corthay Francois, Gubler Oliver
#
#-------------------------------------------------------------------------------
#
# Description: 
#   Updates the file references in the .prj Actel project file
#   and launches the Libero project manager
#
#-------------------------------------------------------------------------------
#
# History:
#   V0.1 : cof 12.2013 -- Initial release
#   V0.2 : guo 08.2014 -- create nonexisting projectDir
#					   -- unified printing
#
################################################################################

use File::Copy qw(copy);
use File::Path qw(mkpath);

$separator = '-' x 79;
$indent = ' ' x 2;
$verbose = 1;

$ENV{TZ} = ''; # needed to be able to run Synplify avoinding license error
my $designerExe = "$ENV{ACTEL_HOME}\\Designer\\bin\\libero.exe";
my $actelFileSpec = $ARGV[0];
$actelFileSpec =~ s/\//\\/g;
$actelFileSpec =~ s/\\\w+\\\.\.//;


#-------------------------------------------------------------------------------
# Project variables
#
                                                              # source directory
my $sourceDir = $actelFileSpec;
$sourceDir =~ s/\.\w+\Z//;
$sourceDir =~ s/\\\w+\Z//;
$sourceDir =~ s/\\actel\Z/\\concat/;
                                                             # generic file spec
my $sourceFile = $actelFileSpec;
$sourceFile =~ s/\.\w+\Z//;
$sourceFile =~ s/\A.+\\//;
                                                             # source file specs
my $vhdlFileSpec = "$sourceDir\\$sourceFile.vhd";
my $pdcFileSpec = "$sourceDir\\$sourceFile.pdc";
                                                                # work directory
my $projectDir = "$ENV{SCRATCH_DIR}\\$ENV{DESIGN_NAME}\\$ENV{ACTEL_WORK_DIR}";
unless (-d $projectDir) {
  mkpath($projectDir) or die "ERROR: Failed to create path: $projectDir";
  print "\n$separator\nCreated directory: $projectDir\n";
}
                                                                # work file spec
$actelWorkFileSpec = "$projectDir\\$sourceFile.prj";

if ($verbose == 1) {
  print "\n$separator\n";
  print "sourceDir: $sourceDir\n";
  print "sourceFile: $sourceFile\n";
  print "vhdlFileSpec: $vhdlFileSpec\n";
  print "pdcFileSpec: $pdcFileSpec\n";
  print "projectDir: $projectDir\n";
}

#-------------------------------------------------------------------------------
# Update paths in the project file
#

if ($verbose == 1) {
  print "\n$separator\n";
  print "Updating file specifications in $actelFileSpec\n";
  print $indent, "temporary file spec: $actelWorkFileSpec\n";
}
my $line;

open(ActelFile, $actelFileSpec) || die "ERROR: Couldn't open $actelFileSpec!";
open(workFile, ">$actelWorkFileSpec");
# while (chop($line = <ActelFile>)) {
while (<ActelFile>) {
  chomp;
  $line = $_;
                                               # replace source path
  if ($line =~ m/DEFAULT_IMPORT_LOC/i) {
    $line =~ s/".*"/"$sourceDir"/;
  }
                                                          # replace project path
  if ($line =~ m/ProjectLocation/i) {
    $line =~ s/".*"/"$projectDir"/;
  }
                                                        # replace VHDL file spec
  if ($line =~ m/VALUE\s".*,hdl"/i) {
    $line =~ s/".*"/"$vhdlFileSpec,hdl"/;
  }
                                                         # replace PDC file spec
  if ($line =~ m/VALUE\s".*\.pdc,pdc"/i) {
    $line =~ s/".*"/"$pdcFileSpec,pdc"/;
  }
  
  print workFile ("$line\n");
}

close(workFile);
close(ActelFile);


#-------------------------------------------------------------------------------
# delete original file and rename temporary file
#
if ($verbose == 1) {
  print "\n$separator\n";
  print "Replace $actelFileSpec with temporary file $actelWorkFileSpec.\n";
}
unlink($actelFileSpec);
#rename($actelWorkFileSpec, $actelFileSpec);
copy($actelWorkFileSpec, $actelFileSpec);


#-------------------------------------------------------------------------------
# read original file, edit and save to temporary file
#

if ($verbose == 1) {
  print "\n$separator\n";
  print "launching $designerExe\n";
  print $indent, "project file spec: $actelWorkFileSpec\n";
}

system("$designerExe $actelWorkFileSpec")or die "ERROR: Failed to launch $actelWorkFileSpec with $designerExe";
